@@ -1,36 +1,77 @@
 # dio.tcl -- implements a database abstraction layer.
 
 # Copyright 2002-2004 The Apache Software Foundation
-
+#
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
 # You may obtain a copy of the License at
-
+#
 #       http://www.apache.org/licenses/LICENSE-2.0
-
+#
 # Unless required by applicable law or agreed to in writing, software
 # distributed under the License is distributed on an "AS IS" BASIS,
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# $Id$
-
 catch {package require Tclx}
 package require Itcl
+package require dio::formatters 1.1
+
+# Command ::rivet::lempty is extensively used within this class but it's
+# defined only when we run DIO from mod_rivet. For convenience we load it
+# here if needed
+
+if {[info commands ::rivet::lempty] == ""} {
+
+    namespace eval ::rivet {
+
+        proc lempty {list} {
+            if {[catch {llength $list} len]} { return 0 }
+            return [expr {$len == 0}]
+        }
+
+    }
+
+}
+
 ##set auto_path [linsert $auto_path 0 [file dirname [info script]]]
 
 namespace eval ::DIO {
 
 proc handle {interface args} {
-    set obj \#auto
+    set obj ::DIO::#auto
+    set tdbc_driver ""
+    if {($interface == "Tdbc") && ([llength $args] > 0)} {
+        set tdbc_driver [lindex $args 0]
+        set args [lreplace $args 0 0]
+    }
+
+    #puts "interface: $interface ($tdbc_driver)"
     set first [lindex $args 0]
-    if {![lempty $first] && [string index $first 0] != "-"} {
+    if {![::rivet::lempty $first] && [string index $first 0] != "-"} {
         set obj  [lindex $args 0]
         set args [lreplace $args 0 0]
     }
     uplevel \#0 package require dio_$interface
-    return [uplevel \#0 ::DIO::$interface $obj $args]
+
+    #puts "tdbc: '$tdbc_driver' obj: '$obj' args 3: '$args'"
+    if {$tdbc_driver == ""} {
+
+        # old connectors based on traditional dbms drivers
+
+        set dio_o [uplevel \#0 ::DIO::$interface $obj $args]
+    } else {
+        set dio_o [uplevel \#0 ::DIO::$interface $obj $tdbc_driver {*}$args]
+    }
+
+    # the Tdbc driver may rename the drive because we created
+    # a set of equivalence classes among DBMS (eg mariadb,sqlite etc)
+    # therefore the class for the interface passed by the constructor
+    # may not exist and it's ::DIO::Tdbc responsability to create it
+
+    $dio_o create_field_formatter
+    return $dio_o
 }
 
 ##
@@ -58,9 +99,8 @@ proc handle {interface args} {
     # quote - given a string, return the same string with any single
     #  quote characters preceded by a backslash
     #
-    method quote {string} {
-        regsub -all {'} $string {\'} string
-        return $string
+    method quote {a_string} {
+        return [DIO::formatters::quote $a_string]
     }
 
     #
@@ -72,11 +112,11 @@ proc handle {interface args} {
     #
     protected method build_select_query {args} {
 
-        set bool AND
-        set first 1
-        set req ""
+        set bool    AND
+        set first   1
+        set req     ""
         set myTable $table
-        set what "*"
+        set what    "*"
 
         # for each argument passed us...
         # (we go by integers because we mess with the index based on
@@ -86,17 +126,17 @@ proc handle {interface args} {
             set elem [lindex $args $i]
 
             switch -- [::string tolower $elem] {
-                "-and" { 
+                "-and" {
                     # -and -- switch to AND-style processing
                     set bool AND 
                 }
 
-                "-or"  { 
+                "-or"  {
                     # -or -- switch to OR-style processing
                     set bool OR 
                 }
 
-                "-table" { 
+                "-table" {
                     # -table -- identify which table the query is about
                     set myTable [lindex $args [incr i]] 
                 }
@@ -170,7 +210,7 @@ proc handle {interface args} {
     protected method build_insert_query {arrayName fields {myTable ""}} {
         upvar 1 $arrayName array
 
-        if {[lempty $myTable]} { set myTable $table }
+        if {[::rivet::lempty $myTable]} { set myTable $table }
         set vals [::list]
         set vars [::list]
         foreach field $fields {
@@ -193,13 +233,14 @@ proc handle {interface args} {
     #
     protected method build_update_query {arrayName fields {myTable ""}} {
         upvar 1 $arrayName array
-        if {[lempty $myTable]} { set myTable $table }
+        if {[::rivet::lempty $myTable]} { set myTable $table }
         set string [::list]
         foreach field $fields {
             if {![info exists array($field)]} { continue }
             lappend string "$field=[makeDBFieldValue $myTable $field $array($field)]"
         }
-        return "update $myTable SET [join $string {,}]"
+
+        return "UPDATE $myTable SET [join $string {,}]"
     }
 
     #
@@ -221,7 +262,7 @@ proc handle {interface args} {
     # variable to the string.
     #
     protected method configure_variable {varName string} {
-        if {[lempty $string]} { return [cget -$varName] }
+        if {[::rivet::lempty $string]} { return [cget -$varName] }
         configure -$varName $string
     }
 
@@ -257,8 +298,8 @@ proc handle {interface args} {
     # values as a list
     ##
     method makekey {arrayName {myKeyfield ""}} {
-        if {[lempty $myKeyfield]} { set myKeyfield $keyfield }
-        if {[lempty $myKeyfield]} {
+        if {[::rivet::lempty $myKeyfield]} { set myKeyfield $keyfield }
+        if {[::rivet::lempty $myKeyfield]} {
             return -code error "No -keyfield specified in object"
         }
         upvar 1 $arrayName array
@@ -371,16 +412,16 @@ proc handle {interface args} {
         set data(-keyfield) $keyfield
         ::array set data $list
 
-        if {[lempty $data(-table)]} {
+        if {[::rivet::lempty $data(-table)]} {
             return -code error -errorcode missing_table "-table not specified in DIO object"
         }
         set $tableVar $data(-table)
 
-        if {[lempty $data(-keyfield)]} {
+        if {[::rivet::lempty $data(-keyfield)]} {
             return -code error -errorcode missing_keyfield "-keyfield not specified in DIO object"
         }
 
-        set $keyVar   $data(-keyfield)
+        set $keyVar $data(-keyfield)
     }
 
     #
@@ -462,11 +503,11 @@ proc handle {interface args} {
     #
     method update {arrayName args} {
         table_check $args
-        upvar 1 $arrayName $arrayName $arrayName array
+        upvar 1 $arrayName $arrayName $arrayName row_a
 
         set key [makekey $arrayName $myKeyfield]
 
-        set fields [::array names array]
+        set fields [::array names row_a]
         set req [build_update_query array $fields $myTable]
         append req [build_key_where_clause $myKeyfield $key]
 
@@ -599,12 +640,31 @@ proc handle {interface args} {
         return [string "select count(*) from $myTable"]
     }
 
-    method makeDBFieldValue {table_name field_name val} {
-        return "'[quote $val]'"
+    public method create_field_formatter {} {
+        set special_fields_formatter [::DIO::formatters::[::string totitle $interface] ::DIO::formatters::#auto]
     }
 
-    method registerSpecialField {table_name field_name type} {
-        set specialFields(${table_name}@${field_name}) $type
+    protected method set_field_formatter {formatter_class} {
+        if {$special_fields_formatter != ""} { $special_fields_formatter destroy }
+        set special_fields_formatter [$formatter_class ::DIO::formatters::#auto]
+    }
+
+    public method build_special_field {table_name field_name val {convert_to {}}} {
+        return [$special_fields_formatter build $table_name $field_name $val $convert_to]
+    }
+
+    public method register_special_field {table_name field_name type} {
+        $special_fields_formatter register $table_name $field_name $type
+    }
+
+    # method kept for compatibility
+
+    public method registerSpecialField {table_name field_name type} {
+        $this register_special_field $table_name $field_name $type
+    }
+
+    public method makeDBFieldValue {table_name field_name val {convert_to {}}} {
+        return [$this build_special_field $table_name $field_name $val $convert_to]
     }
 
     ##
@@ -616,8 +676,8 @@ proc handle {interface args} {
     method exec    {args} {}
     method nextkey {args} {}
     method lastkey {args} {}
-    method now {} {}
-
+    method now     {}     {}
+    method last_inserted_rec {} {}
     ##
     ## Functions to get and set public variables.
     ##
@@ -633,7 +693,7 @@ proc handle {interface args} {
     method host {{string ""}} { return [configure_variable host $string] }
     method port {{string ""}} { return [configure_variable port $string] }
 
-    protected variable specialFields
+    public variable special_fields_formatter ""
 
     public variable interface   ""
     public variable errorinfo   ""
@@ -647,6 +707,12 @@ proc handle {interface args} {
     public variable host        ""
     public variable port        ""
 
+    protected method handle_client_arguments {cargs} { }
+
+    public variable clientargs  "" {
+        handle_client_arguments $clientargs
+    }
+
     public variable keyfield    "" {
         if {[llength $keyfield] > 1 && $autokey} {
             return -code error "Cannot have autokey and multiple keyfields"
@@ -681,7 +747,7 @@ proc handle {interface args} {
     # variable to the string.
     #
     protected method configure_variable {varName string} {
-        if {[lempty $string]} { return [cget -$varName] }
+        if {[::rivet::lempty $string]} { return [cget -$varName] }
         configure -$varName $string
     }
 
@@ -742,7 +808,7 @@ proc handle {interface args} {
 
     method next {type {varName ""}} {
         set return 1
-        if {![lempty $varName]} {
+        if {![::rivet::lempty $varName]} {
             upvar 1 $varName var
             set return 0
         }
@@ -755,7 +821,7 @@ proc handle {interface args} {
             set list $cacheArray($rowid)
         } else {
             set list [$this nextrow]
-            if {[lempty $list]} {
+            if {[::rivet::lempty $list]} {
                 if {$return} { return }
                 set var ""
                 return 0
@@ -789,16 +855,58 @@ proc handle {interface args} {
                 }
                 if {$return} { return $var }
             }
-
+            "-dict" {
+                foreach field $fields elem $list {
+                    lappend var $field $elem
+                }
+                if {$return} { return [dict create {*}$var] }
+            }
             default {
                 incr rowid -1
                 return -code error \
-                    "In-valid type: must be -list, -array or -keyvalue"
+                    "In-valid type: must be -list, -array, -dict or -keyvalue"
             }
         }
-        return [expr [lempty $list] == 0]
+        return [expr [::rivet::lempty $list] == 0]
     }
 
+    public method mkdict {key} {
+
+        set query_res_d [dict create]
+
+        # we check on the first row a value
+        # for the corresponding key exists
+
+        if {[$this next -dict d] > 0} {
+            if {[dict exists $d $key]} {
+
+                set keyvalue [dict get $d $key]
+                dict unset d $key
+
+                dict set query_res_d $keyvalue $d
+            } else {
+
+                return -code error \
+                    "Value for key '$key' not existing in the query results" 
+
+            }
+        }
+
+        # then we proceed with the remaining rows to
+        # be processed
+
+        while {[$this next -dict d]} {
+
+            set keyvalue [dict get $d $key]
+            dict unset d $key
+
+            dict set query_res_d $keyvalue $d 
+        }
+
+        return $query_res_d
+    }
+
+
     method resultid {{string ""}} { return [configure_variable resultid $string] }
     method fields {{string ""}} { return [configure_variable fields $string] }
     method rowid {{string ""}} { return [configure_variable rowid $string] }
@@ -825,4 +933,4 @@ proc handle {interface args} {
 
 } ; ## namespace eval DIO
 
-package provide DIO 1.0
+package provide DIO 1.2.3
