@@ -1,7 +1,6 @@
 #
 # Session - Itcl object for web session management for Rivet
 #
-# $Id$
 #
 
 # Copyright 2004 The Apache Software Foundation
@@ -18,7 +17,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-package provide Session 1.0
+package provide Session 1.1
 package require Itcl
 
 ::itcl::class Session {
@@ -105,15 +104,15 @@ package require Itcl
 
     constructor {args} {
 	eval configure $args
-    	$dioObject registerSpecialField $sessionTable session_update_time NOW
+        $dioObject registerSpecialField $sessionTable session_update_time NOW
 	$dioObject registerSpecialField $sessionTable session_start_time NOW
     }
 
     method status {args} {
-	if {$args == ""} {
-	    return $status
-	}
-	set status $args
+        if {$args == ""} {
+            return $status
+        }
+        set status $args
     }
 
     # get_entropy_bytes - read entropyLength bytes from a random data
@@ -165,11 +164,13 @@ package require Itcl
     method do_garbage_collection {} {
 	debug "do_garbage_collection: performing garbage collection"
 #	set result [$dioObject exec "delete from $sessionTable where timestamp 'now' - session_update_time > interval '$gcMaxLifetime seconds';"]
+        #
+        debug "do_garbage_collection: t1: [$dioObject makeDBFieldValue $sessionTable session_update_time now SECS] t2: [$dioObject makeDBFieldValue $sessionTable session_update_time {} SECS]"
 	set del_cmd "delete from $sessionTable where "
 	append del_cmd [$dioObject makeDBFieldValue $sessionTable session_update_time now SECS]
 	append del_cmd " - [$dioObject makeDBFieldValue $sessionTable session_update_time {} SECS]"
 	append del_cmd " > $gcMaxLifetime"
-	debug "do_garbage_collection: > $del_cmd  <"
+	debug "do_garbage_collection: > $del_cmd <"
 	set result [$dioObject exec $del_cmd]
 	$result destroy
     }
@@ -181,6 +182,8 @@ package require Itcl
     #   performed.
     #
     method consider_garbage_collection {} {
+	debug "consider_garbage_collection: probability: $gcProbability"
+
 	if {rand() <= $gcProbability / 100.0} {
 	    do_garbage_collection
 	}
@@ -347,21 +350,18 @@ package require Itcl
     }
 
     #
-    # stash -
+    # stash - given a packagename and a key-value dictionary of data
+    # the procedure calls method store to data into the cache table using
+    # the dictionary keys as column _key value
     #
     #
 
     method stash {packageName keyvalue_d} {
-
         dict for {key value} $keyvalue_d {
-
             $this store $packageName $key $value
-
         }
-
     }
 
-
     #
     # load - given a package names returns a dictionary storing the key - value pairs for this session
     #
@@ -381,6 +381,7 @@ package require Itcl
     #         when also the key value is specified only the row for that package-key pair is deleted
     #
     method clear {packageName {key ""}} {
+
         if {$key == ""} {
             set keyval      [list [$this id] $packageName]
             set keyfield    {session_id package_}
@@ -475,7 +476,7 @@ package require Itcl
     }
 
     #
-    # is_new_sesion - return a 1 if it's a new session, else a zero if there
+    # is_new_session - return a 1 if it's a new session, else a zero if there
     # were one or more prior pages creating and/or using this session ID
     #
     method is_new_session {} {
